/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWTRACKERMODEL_H
#define MAFWTRACKERMODEL_H

#include <QObject>
#include <QAbstractTableModel>

// column headers
namespace MafwTrackerModelHeaders
{
    static const QString SongId("songId");
    static const QString SongUri("songUri");
    static const QString SongTitle("songTitle");
    static const QString SongLength("songLength");
    static const QString TrackNumber("trackNumber");
    static const QString ArtistId("artistId");
    static const QString ArtistName("artistName");
    static const QString AlbumId("albumId");
    static const QString AlbumName("albumName");
    static const QString AlbumTrackCount("albumTrackCount");
    static const QString AlbumLength("albumLength");
    static const QString AlbumArtistCount("albumArtistCount");
    static const QString AlbumSongs("albumSongs");
    static const QString NumberOfSongs("numberOfSongs");
    static const QString NumberOfAlbums("numberOfAlbums");
    static const QString Genre("genre");
    static const QString Year("year");
    static const QString Favorited("favorited");
    static const QString UsageCounter("usageCounter");
    static const QString Added("added");
    static const QString PlaylistId("playlistId");
    static const QString PlaylistName("playlistName");
    static const QString PlaylistTrackCount("playlistTrackCount");
    static const QString PlaylistLength("playlistLength");
}

/**
  * @brief MafwTrackerModel
  *
  * This model is designed to store data fetched from Tracker with QSPARQL
  *
  */
class MafwTrackerModel  : public QAbstractTableModel
{
    Q_OBJECT

public:
    /**
     * Type of query the model is based on
     */
    enum QueryType
    {
        AllSongs,
        SongsOfAlbum, // needs albumId parameter, no streaming mode support fetch all at once
        SongsOfArtist, // needs artistId parameter
        RecentlyAddedSongs,  // no streaming mode support fetch all at once
        RecentlyPlayedSongs, // no streaming mode support fetch all at once
        MostPlayedSongs, // no streaming mode support fetch all at once
        FavoriteSongs,
        NeverPlayedSongs,
        DownloadedSongs,
        SongsOfPlaylist, // needs playlistId parameter
        SongsOfUnknownArtist,
        SongsOfUnknownAlbum,
        SongsOfUnknownAlbumOfKnownArtist,
        SongsOfUnknownAlbumOfUnknownArtist,
        OneSong, // needs songId parameter, no streaming mode support
        AllAlbums = 100,
        AlbumsOfArtist, // needs artistId parameter
        RecentlyAddedAlbums, // no streaming mode support fetch all at once
        AlbumsOfUnknownArtist, // albums without artist metadata
        AllArtists = 200,
        AllPlaylists = 300,
        RecentlyAddedPlaylists, // no streaming mode support fetch all at once
        Copy = 400 // for internal use
    };

    enum FetchRestResult
    {
        FetchRestNotSupported, // model does not support fetchRest feature at all
        FetchRestNotAllowed, // model is in wrong state
        FetchRest, // do fetch rest now
        FetchRestQueued, // first block ongoing, fetch rest right after that is completed
        FetchAll // dirty state, fetch all and reset model
    };

    /**
     * Type of query the model is based on
     */
    enum SupplementaryQueryType
    {
        UnknownAlbumsOfArtist, // needs artistId parameter, callback slot signature: slotname( int amountOfSongs, int durationOfSongs )
        SongsAndDurationOfArtist // needs artistId parameter, callback slot signature: slotname( int amountOfSongs, int durationOfSongs )
    };

public:
    /**
      * @return True is model is "live" i.e. inital query is completed and updates are not explicitly disabled.
      */
    virtual bool isAutomaticallyUpdated() const = 0;

    /**
      * Disable updates.
      */
    virtual void disableAutomaticUpdates() = 0;

    /**
      * @return True if inital query is completed.
      */
    virtual bool isCompleted() const = 0;

    /**
      * @return True if first block is fetched and changes have happened after that.
      * Calling fetchRest() will cause model reset.
      */
    virtual bool isDirty() const = 0;

    /**
      * Fetches rest items if firstBlockSize!=0 was given to MafwTrackerModelFactory::getModel().
      * @return Result code
      */
    virtual FetchRestResult fetchRest() = 0;

public slots:
    /**
      * Request model to emit dataChanged for given index.
      */
    virtual void emitDataChanged(const QModelIndex& index) = 0;

signals:
    /**
      * Emitted when all results are received.
      */
    void completed();

    /**
    * Emitted if query fails.
    * @param message Engineer readable error message.
    */
    void error(QString message);
};

#endif // MAFWTRACKERMODEL_H
