/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWTRACKERMODELCONNECTION_H
#define MAFWTRACKERMODELCONNECTION_H

#include <QObject>
#include <QVariant>

class QSparqlConnection;

class MafwTrackerModelConnection : public QObject
{
    Q_OBJECT

public:
    /**
      * Enqeueue tracker query. Results or error are delivered to given slots.
      *
      * Data slot signature is
      * void slot( QList<QStringList> rows, bool end, const QVariant& userdata );
      * where 'rows' is result and 'end' parameter indicates if query was completed or there is more to come.
      *
      * Error slot signature is
      * void slot( int code, QString msg, const QVariant& userData );
      * where error 'code' and 'msg' are from QSparql.
      *
      * Example invocation:
      * @code
      * m_conn.queueQuery( mySparql, 5, this, SLOT( processResult(QList<QStringList>,bool,const QVariant&) ), SLOT( queryFailed(int,QString,const QVariant&) ) );
      * @endcode
      *
      * @param query Sparql query
      * @param cols Column count in query (must match to columns in given sparql query)
      * @param receiver Object to receive result or error.
      * @param dataSlot Slot to rceive results
      * @param errorSlot Slot to receive error
      * @param userData Application data that is delivered to callback.
      */
    virtual void queueQuery(QString query, int cols, QObject* receiver,  const char* dataSlot, const char* errorSlot, const QVariant& userData=QVariant()) = 0;

    /**
      * Getter for current tracker indexing status
      * @return active True if miner is working
      */
    virtual bool getTrackerIndexingStatus() const = 0;

    /**
      * If this is set to true then trackerIndexing signal is sent immedately when tracker starts indexing.
      * When false (default) signal is sent only after music related changes are happening.
      */
    virtual void setImmediateIndexingIndication(bool immediate) = 0;


signals:
    /**
      * Signal for added music items
      * @param TrackerId list
      */
    void musicAdded( const QSet<int>& ids );
    /**
      * Signal for deleted music items
      * @param TrackerId list
      */
    void musicDeleted( const QSet<int>& ids );
    /**
      * Signal for modified music items
      * @param TrackerId list
      */
    void musicChanged( const QSet<int>& ids );
    /**
      * Signal for favorite tagged music items
      * @param TrackerId list
      */
    void musicFavorited( const QSet<int>& ids );
    /**
      * Signal for unfavorited music items
      * @param TrackerId list
      */
    void musicUnfavorited( const QSet<int>& ids );
    /**
      * Signal for modified music items of album
      * @param TrackerId list
      */
    void musicOfAlbumChanged( const QSet<int>& ids );
    /**
      * Signal for modified music items of artist
      * @param TrackerId list
      */
    void musicOfArtistChanged( const QSet<int>& ids );
    /**
      * Signal for music items playcount changes
      * @param TrackerId list
      */
    void musicAccessed( const QSet<int>& ids );

    /**
      * Signal for added album items
      * @param TrackerId list
      */
    void albumsAdded( const QSet<int>& ids );
    /**
      * Signal for deleted album items
      * @param TrackerId list
      */
    void albumsDeleted( const QSet<int>& ids );
    /**
      * Signal for modifies album items
      * @param TrackerId list
      */
    void albumsChanged( const QSet<int>& ids );

    /**
      * Signal for added artist items
      * @param TrackerId list
      */
    void artistsAdded( const QSet<int>& ids );
    /**
      * Signal for deleted artist items
      * @param TrackerId list
      */
    void artistsDeleted( const QSet<int>& ids );
    /**
      * Signal for modifies artist items
      * @param TrackerId list
      */
    void artistsChanged( const QSet<int>& ids );

    /**
      * Signal for added playlist items
      * @param TrackerId list
      */
    void playlistsAdded( const QSet<int>& ids );
    /**
      * Signal for deleted playlist items
      * @param TrackerId list
      */
    void playlistsDeleted( const QSet<int>& ids );

    /**
      * Signal tells tracker miner indexing status.
      * @param active True if miner is working
      * @param changes Amount of changes (not valid after restart is true).
      * @param restart If true there has been too many changes to models. All existing live models must be deleted: they are not properly updated anymore.
      * Applications need to get models from facotry again after miner has finished i.e. when it has got this signal with active=false;
      */
    void trackerIndexing( bool active, int changes, bool restart );
};

#endif // MAFWTRACKERMODELCONNECTION_H
