/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWTRACKERMODELFACTORY_H
#define MAFWTRACKERMODELFACTORY_H

#include <QObject>
#include <QSharedPointer>
#include "MafwTrackerModel.h"

#include "MafwTrackerModelConnection.h"
#include "MafwUpdateFetcher.h"


class MafwTrackerModelFactoryPrivate;

/**
  * @brief Create MafwTrackerModels.
  *
  * This factory is used to create models from tracker.
  * Models are automatically updated when changes happen in tracker.
  *
  * @code
  * MafwTrackerModelFactory* factory=new MafwTrackerModelFactory();
  *
  * or for fixed block size model
  *
  * MafwTrackerModelFactory* factory=new MafwTrackerModelFactory( 100 );  // 100 == size of first block
  *
  * factory->init();
  * connect( factory, SIGNAL(initCompleted(bool)), this, SLOT(handleFactoryInitialization(bool) ) );
  * .. in handleFactoryInitialization ..
  * MafwTrackerModel* allAlbums=factory->getModel(MafwTrackerModel::AllAlbums);
  * @endcode
  *
  * @see MafwTrackerModel
  */
class MafwTrackerModelFactory : public QObject
{
    Q_OBJECT

private:
    Q_DISABLE_COPY(MafwTrackerModelFactory)

public:
    /**
      * Constructor
      * streamingBlockSize has to be more than 0 or default value of 1000 will be used
      * @param streamingBlockSize Query results are fetched in chucks of this size when fetchRest() is called.
      */
    MafwTrackerModelFactory( const QString& unknown_album = "", const QString& unknown_artist = "", int streamingBlockSize = 1000 );
    virtual ~MafwTrackerModelFactory();

    /**
      * Initializes the factory asynchronously. initCompleted() is signalled when factory is ready for calling getModel().
      * If initialization fails the whole factory has to be deleted and another instance made to be able to try again.
      * Init can be called only once per factory instance.
      */
    void init();

    /**
      * Request query additional data from tracker
      * @param receiver The object that calls this function
      * @param method The slot that will be called when the query finishes
      * @param queryType The type of query requested
      * @param trackerId Tracker item id parameter is needed in some queries.
      */
    void requestSupplementaryQuery( const QObject * receiver, const char * method, MafwTrackerModel::SupplementaryQueryType queryType, int trackerId = 0 );

    /**
      * Create model. Model will be automatically updated.
      * @param queryType What kind of model to create. MafwTrackerModel::QueryType.
      * @param firstBlockSize Get this many items at first and rest when fetchRest() is called. Default value 0 means fetch all immediately.
      * @param trackerId Tracker item id parameter is needed in some queries.
      * @return Pointer to created model. Ownership transfers. Returns 0 if model can not be created (e.g. the factory is not initialized).
      *
      * @note firstBlockSize must be smaller than streamingBlockSize.
      * @see MafwTrackerModel
      */
    MafwTrackerModel* getModel( MafwTrackerModel::QueryType queryType, int firstBlockSize = 0, int trackerId = 0);

    /**
      * Create copy of model. Song models only are supported.
      * Data in copy will be updated but new items are not added and its not re-sorted. If song is deleted it gets deleted form copy too.
      * If given original model is not completed then data is added to copy until completed and after that shared pointer is released.
      * Favorite tag can be set to tracker via copy model too.
      *
      * @param model Shared pointer to original model. Shared pointer is released after model is completed.
      * @param row If row<0 whole model is copied. If positive row is given then only that row is copied.
      * @return Pointer to created copy model. Ownership transfers. Returns 0 if model can not be created.
      */
    MafwTrackerModel* copyModel(const QSharedPointer<MafwTrackerModel> &model, int row = -1);

    /**
      * Apps can get signals about tracker changes from MafwTrackerModelConnection.
      */
    MafwTrackerModelConnection* trackerConnection();

    /**
      * Apps can get signals about details of additions and updates from MafwUpdateFetchers.
      */
    MafwUpdateFetcher* songUpdateFetcher();
    MafwUpdateFetcher* albumUpdateFetcher();
    MafwUpdateFetcher* artistUpdateFetcher();
    MafwUpdateFetcher* playlistUpdateFetcher();

signals:
    /**
      * Signalled when initialization completes, @see init()
      * @param success True when succeeded.
      */
    void initCompleted( bool success );
    /**
      * Signalled when unknown artists value changes. Artist model has to
      * be created before this is signalled.
      * @param unknownartists True when there are songs without artist information
      */
    void unknownArtistCountChanged( bool unknownartists );
    /**
      * Signalled when unknown albums value changes. Album model has to
      * be created before this is signalled.
      * @param unknownalbums True when there are songs without album information
      */
    void unknownAlbumsCountChanged( bool unknownalbums );

private:
    MafwTrackerModelFactoryPrivate* d;
    QString m_unknown_album;
    QString m_unknown_artist;
};

#endif // MAFWTRACKERMODELFACTORY_H
